/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.geoviewer;

import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import java.awt.Image;
import java.beans.PropertyChangeListener;
import java.util.concurrent.Callable;
import java.util.logging.Logger;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.geometry.Envelope2D;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.openide.util.Exceptions;

/**
 * Base class for GridCoverage2D Geographical objects (Grids) Geometric
 * functions getCentroid() and getBoundingBox() are implemented.
 *
 *
 *
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public abstract class Grid implements Geographical<GridCoverage2D> {
    
    /**
     * Sentinel value for empty spots in the grid, usually rendered transparent.
     * 
     * Type <code>int</code> because it's the lowest supported precision for samples in a {@link java.awt.image.WritableRaster}.
     * As low as possible because it's used as the first entry in {@link org.geotools.styling.ColorMap}s.
     */
    public static final int EMPTY_VALUE = Integer.MIN_VALUE;
    
    private GridCoverage2D internalGrid;
    private Callable<GridCoverage2D> callable;
    private Geometry boundingBox;
    private Point centroid;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.geoviewer.Grid");

    /**
     * Initialize the Grid with a GridCoverage2D Use {@link Grid#Grid(java.util.concurrent.Callable}
     * to postpone grid initialization.
     *
     * @param grid
     */
    public Grid(GridCoverage2D grid) {
        this.internalGrid = grid;
    }

    /**
     * Initialize the Grid with a Callable which returns a GridCoverage2D on
     * call. Grid initialization can take some time. This way initialization can
     * be postponed to the moment it is needed to avoid unnecessary
     * calculations.
     *
     * @param callable
     */
    public Grid(Callable<GridCoverage2D> callable) {
        this.callable = callable;
    }

    private GridCoverage2D getInternalGrid() {
        if (internalGrid == null && callable == null) {
            throw new IllegalStateException("Either the internal grid or the future should not be null");
        }
        if (internalGrid == null) {
            try {
                internalGrid = callable.call();
            } catch (Exception ex) {
                Exceptions.printStackTrace(ex);
            }
        }
        return internalGrid;
    }

    @Override
    public Point getCentroid() {
        if (getInternalGrid() == null) {
            LOGGER.warning("FIXME:getCentroid called while grid is null");
            return null;
        }

        if (centroid == null) {
            Double x = getInternalGrid().getEnvelope2D().getCenterX();
            Double y = getInternalGrid().getEnvelope2D().getCenterY();
            centroid = new Point(new CoordinateArraySequence(new Coordinate[]{new Coordinate(x, y)}), JTSFactoryFinder.getGeometryFactory(null));
        }
        return centroid;
    }

    @Override
    public GridCoverage2D getRenderObject(Envelope envelope) {
        return getInternalGrid();
    }

    @Override
    public Geometry getBoundingBox() {
        if (getInternalGrid() == null) {
            LOGGER.warning("FIXME:getBoundingBox called while grid is null");
            return null;
        }

        if (boundingBox == null) {
            LOGGER.finest("Initializing bounding box for Grid");
            Coordinate[] coordinates = new Coordinate[5];

            Envelope2D e2d = getInternalGrid().getEnvelope2D();

            coordinates[0] = new Coordinate(e2d.x, e2d.y);
            coordinates[1] = new Coordinate(e2d.x + e2d.width, e2d.y);
            coordinates[2] = new Coordinate(e2d.x + e2d.width, e2d.y + e2d.height);
            coordinates[3] = new Coordinate(e2d.x, e2d.y + e2d.height);
            coordinates[4] = new Coordinate(e2d.x, e2d.y);

            GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory(null);
            LinearRing linearRing = new LinearRing(new CoordinateArraySequence(coordinates), geometryFactory);
            boundingBox = new Polygon(linearRing, null, geometryFactory);
        }
        return boundingBox;
    }

    @Override
    public String getIconLabel() {
        return null;
    }

    @Override
    public Image getIcon() {
        return null;
    }

    @Override
    public String getTooltipText() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener listener) {
    }
}
